-- Edited by LJ Sonic to make it work in netgames
-- and fix the performance issues, as well as a few bugs


rawset(_G, "daynightcycle", {})


local dn = daynightcycle


dn.MINUTE = 60 * TICRATE
dn.HOUR = 60 * dn.MINUTE
dn.DAY = 24 * dn.HOUR

dn.MORNING_START = 5*dn.HOUR
dn.MORNING_END = 9*dn.HOUR
dn.EVENING_START = 19*dn.HOUR
dn.EVENING_END = 23*dn.HOUR


dn.time = (dn.MORNING_START + dn.MORNING_END) / 2
dn.frozentime = false

rawset(_G, "musicshift", musicshift or {})
rawset(_G, "musicinvert", musicinvert or {})


local cv_timefactor = CV_RegisterVar({"timefactor", 150, CV_NETVAR, { MIN = 0, MAX = dn.DAY }})


function dn.easeLinear(t, min, max)
	return min + FixedMul(t, max - min)
end

function dn.getTimeDiff(startTime, endTime)
	local delta = endTime - startTime
	if delta < 0 then
		delta = delta + dn.DAY
	end
	return delta
end

function dn.isTimeInRange(time, startTime, endTime)
	return (dn.getTimeDiff(startTime, time) < dn.getTimeDiff(startTime, endTime))
end

function dn.getDayPhase()
	local time = dn.time

	local phase
	local startTime, endTime
	if dn.isTimeInRange(time, dn.MORNING_START, dn.MORNING_END) then
		phase = "morning"
		startTime, endTime = dn.MORNING_START, dn.MORNING_END
	elseif dn.isTimeInRange(time, dn.MORNING_END, dn.EVENING_START) then
		phase = "day"
		startTime, endTime = dn.MORNING_END, dn.EVENING_START
	elseif dn.isTimeInRange(time, dn.EVENING_START, dn.EVENING_END) then
		phase = "evening"
		startTime, endTime = dn.EVENING_START, dn.EVENING_END
	else
		phase = "night"
		startTime, endTime = dn.EVENING_END, dn.EVENING_START
	end

	local ratio = FixedDiv(dn.getTimeDiff(startTime, time), dn.getTimeDiff(startTime, endTime))

	return phase, ratio
end

function dn.isDayHalf()
	local morning = (dn.MORNING_START + dn.MORNING_END) / 2
	local evening = (dn.EVENING_START + dn.EVENING_END) / 2
	return dn.isTimeInRange(dn.time, morning, evening)
end

local function updateTime()
	if not dn.frozentime
		dn.time = ($ + cv_timefactor.value) % dn.DAY
	end

	if not (mapheaderinfo[gamemap].spacemap)
		local sky
		local phase, ratio = dn.getDayPhase()
		if phase == "morning" then
			sky = dn.easeLinear(ratio, 5009, 5001)
		elseif phase == "day" then
			sky = 5001
		elseif phase == "evening" then
			sky = dn.easeLinear(ratio, 5001, 5009)
		else
			sky = 5009
		end

		P_SetupLevelSky(sky)
	end

	dn.updateLight()
end

local function musicPlayerThink(p)
	if musicshift[S_MusicName()] and not dn.isDayHalf() and not (p.mo.QuitTryingToChangeTheFuckingMusicYouCunt)
		S_ChangeMusic(musicshift[S_MusicName()], true, p, 0, 0, MUSICRATE, MUSICRATE)
		p.mo.QuitTryingToChangeTheFuckingMusicYouCunt = TICRATE*2
	elseif musicinvert[S_MusicName()] and dn.isDayHalf() and not (p.mo.QuitTryingToChangeTheFuckingMusicYouCunt)
		S_ChangeMusic(musicinvert[S_MusicName()], true, p, 0, 0, MUSICRATE, MUSICRATE)
		p.mo.QuitTryingToChangeTheFuckingMusicYouCunt = TICRATE*2
	end

	if (p.mo.QuitTryingToChangeTheFuckingMusicYouCunt) and (p.mo.QuitTryingToChangeTheFuckingMusicYouCunt > 0)
		p.mo.QuitTryingToChangeTheFuckingMusicYouCunt = $-1
	end

	--print(S_MusicName())
	--end
end


addHook("ThinkFrame", function()
	updateTime()
end)

addHook("MapChange", function()
	dn.resetLight()
end)

addHook("PlayerThink", function(p)
	if not (p.mo and p.mo.valid and p.mo.health) return end
	dn.werewolfPlayerThink(p)
	musicPlayerThink(p)
end)

COM_AddCommand("time", function(p, hours, minutes, seconds)
	if hours then
		hours = hours:lower()
	end

	if hours == "day" then
		dn.time = dn.MORNING_END
		return
	elseif hours == "night" then
		dn.time = dn.EVENING_END
		return
	elseif hours == "morning" then
		dn.time = dn.MORNING_START
		return
	elseif hours == "evening" then
		dn.time = dn.EVENING_START
		return
	end

	hours = tonumber($)
	if hours == nil then
		CONS_Printf(p, "Syntax: time <hours> [minutes [seconds]]\n        time {day|night|morning|evening}")
		return
	end
	if hours < 0 or hours > 23 then
		CONS_Printf(p, "Number of hours has to range between 0 and 23.")
		return
	end

	minutes = tonumber($)
	if minutes == nil then
		minutes, seconds = 0, 0
	end
	if minutes < 0 or minutes > 59 then
		CONS_Printf(p, "Number of minutes has to range between 0 and 59.")
		return
	end

	seconds = tonumber($)
	if seconds == nil then
		seconds = 0
	end
	if seconds < 0 or seconds > 59 then
		CONS_Printf(p, "Number of seconds has to range between 0 and 59.")
		return
	end

	dn.time = dn.HOUR * hours + dn.MINUTE * minutes + TICRATE * seconds
end, COM_ADMIN)

/*COM_AddCommand("dndebug_settime", function(player, arg)
	if dn.currenttime != nil and arg 
		dn.currenttime = tonumber(arg)
		if arg == 0 or arg == 90
			dn.inbetweentime = dn.cycle
		else
			dn.inbetweentime = 0
		end
	end
end, COM_ADMIN)

COM_AddCommand("dndebug_timescale", function(player, arg)
	if timescale != nil and arg 
		timescale = tonumber(arg)
	end
	print(timescale)
end, COM_ADMIN)

COM_AddCommand("dndebug_timedir", function(player, arg)
	if dn.reversedir != nil and arg 
		dn.reversedir = arg
	end
end, COM_ADMIN)

COM_AddCommand("dndebug_freezetime", function(player, arg)
	if dn.frozentime != nil and arg
		if arg == "true" or arg == "on" or arg == "1"
			dn.frozentime = true
			print("time frozen")
		elseif  arg == "false" or arg == "off" or arg == "0"
			dn.frozentime = false
			print("time unfrozen")
		end
	end
end, COM_ADMIN)*/

local yourmom

addHook("PreThinkFrame", function(r)
	yourmom = S_GetMusicPosition()
end)

addHook("MusicChange", function(om, nm)
	if not (consoleplayer and consoleplayer.mo and consoleplayer.mo.health) return end

	if dn.isDayHalf()
		if musicinvert[nm]
			return musicinvert[nm], 0, true, yourmom
		end
	else
		if musicshift[nm]
			return musicshift[nm], 0, true, yourmom
		end
	end
end)

addHook("NetVars", function(n)
	dn.time = n($)
	dn.frozentime = n($)
end)
