local dn = daynightcycle


dn.DAY_LIGHT = 255


dn.light = 255
dn.oldLight = dn.light

local origSectorLights
local precalculatedLightLevels -- Only used locally


dn.cv_nightlight = CV_RegisterVar{
	name = "nightlight",
	defaultvalue = 147, --128,
	flags = CV_NETVAR|CV_CALL,
	PossibleValue = { MIN = 0, MAX = 255 }, 
	func = function()
		precalculatedLightLevels = nil
	end
}


local function getLightInTimeRange(time, startTime, endTime, startLight, endLight)
	local ratio = FixedDiv(dn.getTimeDiff(startTime, time), dn.getTimeDiff(startTime, endTime))
	return dn.easeLinear(ratio, startLight, endLight)
end

function dn.getSunLight(time)
	local nightLight = dn.cv_nightlight.value
	local phase = dn.getDayPhase()

	if phase == "morning" then
		return getLightInTimeRange(time, dn.MORNING_START, dn.MORNING_END, nightLight, dn.DAY_LIGHT)
	elseif phase == "day" then
		return dn.DAY_LIGHT
	elseif phase == "evening" then
		return getLightInTimeRange(time, dn.EVENING_START, dn.EVENING_END, dn.DAY_LIGHT, nightLight)
	else
		return nightLight
	end
end

local function initialiseSectorLights()
	origSectorLights = {}
	for i = 0, #sectors - 1 do
		origSectorLights[i] = sectors[i].lightlevel
	end
end

local function precalculateLightLevels()
	local minLight = 147 --dn.cv_nightlight.value
	local lightRange = 255 - minLight

	precalculatedLightLevels = {}

	for light = 0, 255 do
		local lightLevels = {}

		for origLight = 0, 255 do
			if origLight > minLight then
				lightLevels[origLight] = max(origLight - (255 - light), minLight)

				--local ratio = FixedMul(origLight - minLight, lightRange)
				--lightLevels[origLight] = dn.easeLinear(ratio, minLight * FU, light * FU) / FU
			else
				lightLevels[origLight] = origLight
			end
		end

		precalculatedLightLevels[light] = lightLevels
	end
end

local function updateSectors()
	if not origSectorLights then
		initialiseSectorLights()
	end

	if not precalculatedLightLevels then
		precalculateLightLevels()
	end

	local precalculatedLightLevels = precalculatedLightLevels[dn.light]

	-- Store globals into locals to make the loop much faster
	local sectors = sectors
	local origSectorLights = origSectorLights

	for i = 0, #origSectorLights do
		sectors[i].lightlevel = precalculatedLightLevels[origSectorLights[i]]

		/*local s = sectors[i]
		local origLight = origSectorLights[i]

		if origLight > light
			s.lightlevel = light
		else
			s.lightlevel = origLight
		end*/
	end
end

function dn.resetLight()
	dn.light = 255
	dn.oldLight = dn.light
	origSectorLights = nil
end

function dn.updateLight()
	dn.light = dn.getSunLight(dn.time)
	if gamemap == 13 then -- Special case for ACZ1, I guess?
		dn.light = 128
	end

	if dn.light ~= dn.oldLight then
		updateSectors()
	end

	dn.oldLight = dn.light
end


addHook("NetVars", function(n)
	dn.light = n($)
	dn.oldLight = n($)
	origSectorLights = n($)
end)
